#include "config.h"

#include <gtkmm/statusbar.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/fixed.h>
#include <gtkmm/main.h>

#include "main_form.h"
// the rest, backend, etc ...
#include "workbench/wb_context_ui.h"
#include "model/wb_model_diagram_form.h"
#include "workbench/wb_overview.h"
#include "model/wb_overview_physical.h"
#include "linux_utilities/plugin_editor_base.h"
#include "linux_utilities/form_view_base.h"

#include "diagram_size_form.h"
#include "info_box.h"

#include "mdc.h"

#include "gtk_helpers.h"

#include "model_panel.h"
#include "model_diagram_panel.h"
#include "overview_panel.h"
#include "find_panel.h"

#include "linux_utilities/gtk_helpers.h"
#include "linux_utilities/image_cache.h"

#include "linux_utilities/toolbar_manager.h"
#include "sqlide/sql_editor_fe.h"

#include "base/string_utilities.h"
#include "mforms/../gtk/lf_menubar.h"
#include "mforms/../gtk/lf_toolbar.h"

using base::strfmt;


#ifdef EDITION_OSS
# define SPLASH_IMAGE "wb_splashscreen_oss.png"
#else
# define SPLASH_IMAGE "wb_splashscreen_se.png"
#endif


static void set_window_icons(Gtk::Window *window)
{
  std::vector<Glib::RefPtr<Gdk::Pixbuf> > icons;

  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbenchDoc-16.png", false));
  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbenchDoc-24.png", false));
  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbenchDoc-32.png", false));
  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbenchDoc-48.png", false));
  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbenchDoc-64.png", false));
  icons.push_back(ImageCache::get_instance()->image_from_filename("MySQLWorkbenchDoc-128.png", false));

  window->set_icon_list(icons);
}


//------------------------------------------------------------------------------
// @ctx is passed from Program class, see Program::Program
MainForm::MainForm(wb::WBContextUI* ctx)
  : _wbui_context(ctx),
    _exiting(false)
{
  setup_mforms_app();
  
  bec::GRTManager *grtm= _wbui_context->get_wb()->get_grt_manager();
  _ui= new MGGladeXML(grtm->get_data_file_path("wb.glade").c_str(), "", "");
  _toolbar_manager = new ToolbarManager(ctx->get_command_ui());

  _accel_group = get_mainwindow()->get_accel_group();
  if (!_accel_group)
  {
    _accel_group= Gtk::AccelGroup::create();
    get_mainwindow()->add_accel_group(_accel_group);
  }
  mforms::gtk::set_accel_group(_accel_group);

  get_mainwindow()->signal_delete_event().connect(sigc::mem_fun(this, &MainForm::close_window));
  get_mainwindow()->signal_set_focus().connect(sigc::mem_fun(this, &MainForm::on_focus_widget));

  get_mainwindow()->set_title("MySQL Workbench");
  
  //_accel_group= Gtk::AccelGroup::create();
  _menu_manager = new MenuManager(ctx->get_command_ui(), _accel_group);
  //get_mainwindow()->add_accel_group(_accel_group);

  _model_panel= 0;

  _find_panel= 0;

  get_upper_note()->signal_switch_page().connect(sigc::mem_fun(*this, &MainForm::switch_page));
  
  Gtk::Statusbar *status = 0;
  _ui->get("statusbar1", &status);
  if (status)
  {
    status->pack_end(_progress_bar, false, false);
    _progress_bar.show();
  }

  _gui_locked= false;

  //  switch_page(0, 0);
}

//------------------------------------------------------------------------------
MainForm::~MainForm()
{
  notify_callbacks(); // disconnect all signals
  

  delete _menu_manager;
  _menu_manager= 0;
  delete _toolbar_manager;
  delete _model_panel;

  delete _ui;
  _ui= 0;
}

//------------------------------------------------------------------------------

void MainForm::register_form_view_factory(const std::string &name, FormViewFactory factory)
{
  _form_view_factories[name]= factory;
}

//------------------------------------------------------------------------------

static void close_plugin(PluginEditorBase *editor, wb::WBContext *wb)
{
  wb->close_gui_plugin(dynamic_cast<GUIPluginBase*>(editor));
}

//------------------------------------------------------------------------------
bool MainForm::close_window(GdkEventAny *ev)
{
  if (_wbui_context->request_quit())
  {
    get_mainwindow()->hide();
    _wbui_context->perform_quit();
  }
  return true; // true means stop processing the event
}


//------------------------------------------------------------------------------
void MainForm::on_focus_widget(Gtk::Widget *w)
{
  if (_ui)
  {
    void *data = 0;
    bec::UIForm* form = 0;
    
    while (!data && w)
    {
      data = w->get_data("uiform");
      if (data)
      {
        form = reinterpret_cast<bec::UIForm*>(data);
        break;
      }
      w = w->get_parent();
    }
    
    if (form)
      _wbui_context->set_active_form(form);
  }

  _wbui_context->get_command_ui()->signal_validate_edit_menu_items().emit();

  // for updating the Edit menu
 //!!! Gtk::Alignment* menu_holder;
 //!!! _menu_manager->rebuild(_ui->get("main_menu_box", &menu_holder));
}

//------------------------------------------------------------------------------

void MainForm::setup_ui()
{  
  set_window_icons(get_mainwindow());

  register_commands();
  
  reset_layout();

  switch_page(0, 0);
}


//------------------------------------------------------------------------------
void MainForm::show()
{
  get_mainwindow()->show();
}


//------------------------------------------------------------------------------
Gtk::Window* MainForm::get_mainwindow() const
{
  Gtk::Window *win = 0;
  _ui->get("wb_main_window", &win);

  return win;
}


Gtk::Notebook *MainForm::get_upper_note() const
{
  Gtk::Notebook *note= 0;
  _ui->get("model_tabs", &note);
  return note;
}


//------------------------------------------------------------------------------
static bool change_status(Gtk::Statusbar *status, const std::string& text)
{
  status->pop();
  status->push(text);
  return false;
}


void MainForm::show_status_text_becb(const std::string& text)
{
  Gtk::Statusbar *status = 0;

  _ui->get("statusbar1", &status);

  if (_wbui_context->get_wb()->get_grt_manager()->in_main_thread())
    change_status(status, text);
  else
    // execute when idle in case we're being called from the worker thread
    Glib::signal_idle().connect(sigc::bind<Gtk::Statusbar*,std::string>(sigc::ptr_fun(change_status), status, text));
}


//------------------------------------------------------------------------------
bool MainForm::show_progress_becb(const std::string& title, const std::string& status, float pct)
{
  if (pct < 0.0)
    pct = 0.0;

  _progress_bar.set_fraction(pct);
  _progress_bar.set_text(status);

  return true;
}


//------------------------------------------------------------------------------
NativeHandle MainForm::open_plugin_becb(bec::GRTManager* mgr, grt::Module *grtmodule,
                                        const std::string& shlib, const std::string& editor_class,
                                        grt::BaseListRef args, bec::GUIPluginFlags flags)
{
  GUIPluginCreateFunction create_function= 0;
  std::string path= grtmodule->path();
  std::string full_path= path.substr(0, path.rfind('/')+1)+shlib;

  // if not forcing a new window creation, check if there's an editor already open for it
  if (!(flags & bec::ForceNewWindowFlag) && !(flags & bec::StandaloneWindowFlag))
  {
    std::vector<NativeHandle> handles=
      mgr->get_plugin_manager()->get_similar_open_plugins(grtmodule, editor_class, args);

    if (!handles.empty())
    {
      GUIPluginBase* guiplugin = reinterpret_cast<GUIPluginBase*>(handles[0]);
      // try to reuse the plugin
      PluginEditorBase *editor= dynamic_cast<PluginEditorBase*>(guiplugin);
 
      if (flags & bec::StandaloneWindowFlag)
      {
        add_plugin_form(editor);
      }
      else
      {
        // add it to the bottom panel of the current tab
        bring_plugin_pane(editor);
      }
      if (editor && editor->can_close() && editor->switch_edited_object(mgr, args))
      {
        mgr->get_plugin_manager()->forget_gui_plugin_handle(handles[0]);
        return handles[0];
      }
    }
  }

  // lookup for the editor_class symbol (create<class>) in the shlib
  GModule *module= g_module_open(full_path.c_str(), (GModuleFlags)G_MODULE_BIND_LOCAL);
  if (!module)
  {
    g_warning("Could not open editor shared object '%s'", full_path.c_str());
    return 0;
  }

  if (!g_module_symbol(module, ("create"+editor_class).c_str(), (void**)&create_function)
      || !create_function)
  {
    g_warning("UI creation function '%s' not found in %s",
              ("create"+editor_class).c_str(), full_path.c_str());
    g_module_close(module);
    return 0;
  }

  // call the initializer with arguments
  GUIPluginBase *object= (*create_function)(grtmodule, mgr, args);
  if (!object)
  {
    g_warning("UI creation function from %s returned 0", full_path.c_str());
    g_module_close(module);
    return 0;
  }

  g_module_close(module);
  
  PluginEditorBase *editor= dynamic_cast<PluginEditorBase*>(object);
  if (editor)
  {
    if (flags & bec::StandaloneWindowFlag)
    {
      add_plugin_form(editor);
    }
    else
    {
      // add it to the bottom panel
      add_plugin_pane(editor);
    }
  }
  else
  {
    //object->execute();
  }

  // return the pointer
  return reinterpret_cast<NativeHandle>(object);
}


//------------------------------------------------------------------------------
void MainForm::show_plugin_becb(NativeHandle handle)
{
  GUIPluginBase *plugin= reinterpret_cast<GUIPluginBase*>(handle);

//  plugin->show_plugin();

///  if (dynamic_cast<PluginEditorBase*>(plugin))
///    show_lower_tab(dynamic_cast<PluginEditorBase*>(plugin));
}


//------------------------------------------------------------------------------
void MainForm::hide_plugin_becb(NativeHandle handle)
{
//  reinterpret_cast<GUIPluginBase*>(handle)->hide_plugin();
}


void MainForm::call_find()
{
  if (_wbui_context->get_active_context(true) != WB_CONTEXT_QUERY)
  {
    if (validate_find_replace())
      call_find_replace();
  }

  Gtk::Box *box;
 // _toolbar_manager->focus_searchbox(_ui->get("main_toolbar", &box));
#warning missing here
}


void MainForm::call_undo()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->undo();
      return;
    }
  }

  if (_wbui_context->get_active_main_form() && _wbui_context->get_active_main_form()->can_undo())
    _wbui_context->get_active_main_form()->undo();
}


void MainForm::call_redo()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->redo();
      return;
    }
  }

  if (_wbui_context->get_active_main_form() && _wbui_context->get_active_main_form()->can_redo())
    _wbui_context->get_active_main_form()->redo();
}


void MainForm::call_copy()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->copy();
      return;
    }
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_copy())
    _wbui_context->get_active_form()->copy();
  else
  {
    if (dynamic_cast<Gtk::Editable*>(focused))
      dynamic_cast<Gtk::Editable*>(focused)->copy_clipboard();
  }
}


void MainForm::call_cut()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->copy();
      sqled->delete_();
      return;
    }
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_cut())
    _wbui_context->get_active_form()->cut();
  else
  {
    if (dynamic_cast<Gtk::Editable*>(focused))
      dynamic_cast<Gtk::Editable*>(focused)->cut_clipboard();
  }
}


void MainForm::call_paste()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->paste();
      return;
    }
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_paste())
    _wbui_context->get_active_form()->paste();
  else
  {
    if (dynamic_cast<Gtk::Editable*>(focused))
      dynamic_cast<Gtk::Editable*>(focused)->paste_clipboard();
  }
}


void MainForm::call_delete()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->delete_();
      return;
    }
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_delete())
    _wbui_context->get_active_form()->delete_selection();
  else
  {
    if (dynamic_cast<Gtk::Editable*>(focused))
      dynamic_cast<Gtk::Editable*>(focused)->delete_selection();
  }
}

void MainForm::call_select_all()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      sqled->select_all();
      return;
    }
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_select_all())
    _wbui_context->get_active_form()->select_all();
  else
  {
    if (dynamic_cast<Gtk::Editable*>(focused))
      dynamic_cast<Gtk::Editable*>(focused)->select_region(0, -1);
  }
}

void MainForm::call_search()
{
  FormViewBase *form = reinterpret_cast<FormViewBase*>(get_upper_note()->get_nth_page(get_upper_note()->get_current_page())->get_data("FormViewBase"));
  if (form)
    form->find_text(_toolbar_manager->get_search_text());
}


bool MainForm::validate_find()
{
  std::string context = _wbui_context->get_active_context();
  
  if (context == WB_CONTEXT_MODEL || context == WB_CONTEXT_QUERY || context == WB_CONTEXT_PHYSICAL_OVERVIEW)
    return true;
  return validate_find_replace();
}

bool MainForm::validate_undo()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return sqled->can_undo();
  }

  bec::UIForm *form = _wbui_context->get_active_main_form();
  if (form)
    return form->can_undo();
  
  return false;
}

bool MainForm::validate_redo()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return sqled->can_redo();
  }

  bec::UIForm *form = _wbui_context->get_active_main_form();
  if (form)
    return form->can_redo();
  
  return false;
}

bool MainForm::validate_copy()
{
  bool ret = false;

  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return sqled->has_selection();
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_copy())
    ret = true;
  else
  {
    Gtk::Editable *edit= dynamic_cast<Gtk::Editable*>(focused);
    int s, e;
    if (edit && edit->get_selection_bounds(s, e))
      ret = true;
  }

  return ret;
}

bool MainForm::validate_cut()
{
  bool ret = false;

  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return sqled->has_selection() && sqled->is_editable();
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_copy())
    ret = true;
  else
  {
    Gtk::Editable *edit= dynamic_cast<Gtk::Editable*>(get_mainwindow()->get_focus());
    int s, e;
    if (edit && edit->get_selection_bounds(s, e) && edit->get_editable())
      ret = true;
  }
  return ret;
}

bool MainForm::validate_paste()
{
  bool ret = false;

  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return sqled->is_editable();
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_paste())
    ret = true;
  else
  {
    Gtk::Editable *edit= dynamic_cast<Gtk::Editable*>(get_mainwindow()->get_focus());
    if (edit && edit->get_editable())
      ret = true;
  }

  return ret;
}

bool MainForm::validate_delete()
{
  bool ret = false;

  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return sqled->is_editable();
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_delete())
    ret = true;
  else
  {
    Gtk::Editable *edit= dynamic_cast<Gtk::Editable*>(get_mainwindow()->get_focus());
    int s, e;
    if (edit && edit->get_selection_bounds(s, e) && edit->get_editable())
      ret = true;
  }
  return ret;
}


bool MainForm::validate_select_all()
{
  bool ret = false;
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return true;
  }

  if (_wbui_context->get_active_form() && _wbui_context->get_active_form()->can_select_all())
    ret = true;
  else
  {
    Gtk::Editable *edit= dynamic_cast<Gtk::Editable*>(focused);
    if (edit)
      ret = true;
  }

  return ret;
}

bool MainForm::validate_search()
{
  if (get_mainwindow()->get_focus() && IS_SCINTILLA(get_mainwindow()->get_focus()->gobj()))
    return true;
  return true;
}


bool MainForm::find_callback(const std::string &search, const std::string &replace, 
                             mforms::SearchFlags flags, SqlEditorFE *editor)
{
  if (flags&mforms::SearchAll && flags&mforms::SearchDoReplace)
  {
    for (;;)
    {
      editor->find_text(search, 
                        (flags&mforms::SearchMatchCase),
                        (flags&mforms::SearchMatchWholeWord));

      if (editor->has_selection())
        editor->insert_text(replace);
      else
        break;
    }
  }
  else
  {
    if (flags&mforms::SearchDoReplace)
    {
      if (editor->has_selection())
        editor->insert_text(replace);
    }
    else
      editor->find_text(search, 
                        (flags&mforms::SearchMatchCase),
                        (flags&mforms::SearchMatchWholeWord));
  }
  return false;
}


void MainForm::call_find_replace()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();
  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
    {
      mforms::SearchReplace dialog;
      
      dialog.set_callback(sigc::bind(sigc::mem_fun(this, &MainForm::find_callback), sqled));
      dialog.show(true, mforms::SearchNone, true);
    }
  }
}


bool MainForm::validate_find_replace()
{
  Gtk::Widget *focused= get_mainwindow()->get_focus();

  if (focused)
  {
    SqlEditorFE *sqled = SqlEditorFE::from(focused);
    if (sqled)
      return true;
  }

  return false;
}


void MainForm::register_commands()
{
  std::list<std::string> commands;

  //  commands.push_back("overview.mysql_model");
  commands.push_back("show_about");
  commands.push_back("diagram_size");
  //  commands.push_back("view_model_navigator");
  //  commands.push_back("view_catalog");
  //  commands.push_back("view_layers");
  //  commands.push_back("view_user_datatypes");
  //  commands.push_back("view_object_properties");
  //  commands.push_back("view_object_description");
  //  commands.push_back("view_undo_history");
  commands.push_back("reset_layout");
  commands.push_back("show_find_dialog");
  commands.push_back("wb.page_setup");
  commands.push_back("closetab");
  commands.push_back("wb.next_tab");
  commands.push_back("wb.back_tab");
  //  commands.push_back("help_index");
  //  commands.push_back("help_version_check");
  commands.push_back("wb.sidebarHide");

  _wbui_context->get_command_ui()->add_frontend_commands(commands);

  _wbui_context->get_command_ui()->add_builtin_command("search_replace",
                                               sigc::mem_fun(this, &MainForm::call_find_replace),
                                               sigc::mem_fun(this, &MainForm::validate_find_replace));

  _wbui_context->get_command_ui()->add_builtin_command("find",
                                               sigc::mem_fun(this, &MainForm::call_find),
                                               sigc::mem_fun(this, &MainForm::validate_find));
  _wbui_context->get_command_ui()->add_builtin_command("undo",
                                               sigc::mem_fun(this, &MainForm::call_undo),
                                               sigc::mem_fun(this, &MainForm::validate_undo));
  _wbui_context->get_command_ui()->add_builtin_command("redo",
                                               sigc::mem_fun(this, &MainForm::call_redo),
                                               sigc::mem_fun(this, &MainForm::validate_redo));
  _wbui_context->get_command_ui()->add_builtin_command("copy",
                                               sigc::mem_fun(this, &MainForm::call_copy),
                                               sigc::mem_fun(this, &MainForm::validate_copy));
  _wbui_context->get_command_ui()->add_builtin_command("cut",
                                               sigc::mem_fun(this, &MainForm::call_cut),
                                               sigc::mem_fun(this, &MainForm::validate_cut));
  _wbui_context->get_command_ui()->add_builtin_command("paste",
                                               sigc::mem_fun(this, &MainForm::call_paste),
                                               sigc::mem_fun(this, &MainForm::validate_paste));
  _wbui_context->get_command_ui()->add_builtin_command("delete",
                                               sigc::mem_fun(this, &MainForm::call_delete),
                                               sigc::mem_fun(this, &MainForm::validate_delete));
  _wbui_context->get_command_ui()->add_builtin_command("selectAll",
                                               sigc::mem_fun(this, &MainForm::call_select_all),
                                               sigc::mem_fun(this, &MainForm::validate_select_all));
  _wbui_context->get_command_ui()->add_builtin_command("searchbox",
                                               sigc::mem_fun(this, &MainForm::call_search),
                                               sigc::mem_fun(this, &MainForm::validate_search));

}

//------------------------------------------------------------------------------
void MainForm::perform_command_becb(const std::string& command)
{
  if (command == "reset_layout")
    reset_layout();
  else if (command == "show_about")
    show_about();
  else if (command == "diagram_size")
    show_diagram_options();
  else if (command == "show_find_dialog")
    show_find();
  else if (command == "wb.page_setup")
    show_page_setup();
  else if (command == "closetab")
    close_active_tab();
  else if (command == "wb.next_tab")
    get_upper_note()->next_page();
  else if (command == "wb.back_tab")
    get_upper_note()->prev_page();
  /*
   // Help
   else if (command == "help_index")
   Help.ShowHelp(null, System.IO.Path.Combine(Application.StartupPath, "MySQLWorkbench.chm"));
   else if (command == "help_version_check")
   Program.CheckForNewVersion();
   */
  else if (command == "wb.sidebarHide")
  {
    FormViewBase *form = get_active_pane();
    if (form)
      form->toggle_sidebar();
  }
  else
  {
    g_message("Command '%s' not handled!\n", command.c_str());
  }
}

//------------------------------------------------------------------------------
mdc::CanvasView* MainForm::create_view_becb(const model_DiagramRef &diagram)
{
  ModelDiagramPanel *model_panel= Gtk::manage(ModelDiagramPanel::create(_wbui_context));

  model_panel->set_close_editor_callback(sigc::bind(sigc::ptr_fun(close_plugin), _wbui_context->get_wb()));

  _signal_close_editor.connect(sigc::hide_return(sigc::mem_fun(model_panel, &ModelDiagramPanel::close_editors_for_object)));

  model_panel->init(diagram.id());

  add_form_pane(model_panel, !model_panel->get_diagram_form()->is_closed());
 // model_panel->set_data("model_panel", model_panel);


  return model_panel->get_canvas();
}


//------------------------------------------------------------------------------
void MainForm::destroy_view_becb(mdc::CanvasView* view)
{
  Gtk::Notebook *note= get_upper_note();

  if (!_wbui_context->get_wb()->get_grt_manager()->in_main_thread())
    G_BREAKPOINT();
  
  for (int c= note->get_n_pages(), i= 1; i < c; i++)
  {
    FormViewBase *form = reinterpret_cast<FormViewBase*>(note->get_nth_page(i)->get_data("FormViewBase"));
    ModelDiagramPanel *model_panel = dynamic_cast<ModelDiagramPanel*>(form);
    //ModelDiagramPanel *model_panel = reinterpret_cast<ModelDiagramPanel*>(note->get_nth_page(i)->get_data("model_panel"));

    if (model_panel && model_panel->get_canvas() == view)
    {
      note->remove_page(i);
      break;
    }
  }
}

//------------------------------------------------------------------------------
void MainForm::switched_view_becb(mdc::CanvasView* view)
{
  Gtk::Notebook *note= get_upper_note();

  bec::UIForm *view_form= _wbui_context->get_wb()->get_model_context()->get_diagram_form(view);

  for (int c= note->get_n_pages(), i= 1; i < c; i++)
  {
    if (note->get_nth_page(i)->get_data("uiform") == view_form)
    {
      note->get_nth_page(i)->show();
      note->set_current_page(i);
      break;
    }
  }
}

//------------------------------------------------------------------------------

void MainForm::create_main_form_view_becb(const std::string &name, boost::shared_ptr<bec::UIForm> form)
{
  FormViewBase *view;
  
  if (_form_view_factories.find(name) != _form_view_factories.end())
  {
    view= _form_view_factories[name](form);
    add_form_pane(view);
  }
  else
  {
    throw std::runtime_error("Form type not supported.");
  }
}

//------------------------------------------------------------------------------

void MainForm::destroy_main_form_view_becb(bec::UIForm* form)
{
}


//------------------------------------------------------------------------------
void MainForm::tool_changed_becb(mdc::CanvasView* view)
{
  ModelDiagramPanel *panel= get_panel_for_view(view);
  if (panel)
  {
    // update cursor in canvas
    panel->update_tool_cursor();
  }
}

const char* RefreshTypeStr[] = {
  "RefreshNeeded",
    "RefreshNothing",
    "RefreshSchemaList",
    "RefreshSchema",
    "RefreshNewDiagram",
    "RefreshSelection",
    "RefreshMessages",
    "RefreshCloseEditor",
    "RefreshNewModel",
    "RefreshOverviewNodeInfo",
    "RefreshOverviewNodeChildren",
    "RefreshCatalogTreeNode",
    "RefreshViewName",
    "RefreshDocument",
    "RefreshCloseDocument",
    "RefreshZoom",
    "RefreshTimer",
    "RefreshFinishEdit"
};

//------------------------------------------------------------------------------


void MainForm::refresh_gui_becb(wb::RefreshType type, const std::string& arg_id, NativeHandle arg_ptr)
{
  if (_exiting)
    return;
    
//  g_message("%s: %i %s: arg=%s, arg_ptr=%p", __FUNCTION__, type, RefreshTypeStr[type], arg_id.c_str(), arg_ptr);
  // _overview is OverviewPanel
  switch (type)
  {
  case wb::RefreshNeeded:
    {
      Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(_wbui_context->get_wb(), &wb::WBContext::flush_idle_tasks), false));
      break;
    }

  case wb::RefreshCloseDocument:
    prepare_close_document();

    _wbui_context->get_wb()->flush_idle_tasks();

    _wbui_context->get_wb()->close_document_finish();

    handle_model_closed();
    
    get_mainwindow()->set_title(_wbui_context->get_title());
    break;

  case wb::RefreshNothing:
    break;

  case wb::RefreshSchemaList:
  case wb::RefreshSchema:
    {
      ModelDiagramPanel *panel = dynamic_cast<ModelDiagramPanel*>(get_active_pane());
      if (panel)
        panel->refresh_catalog();
      break;
    }
  case wb::RefreshNewDiagram:
    {
      ModelDiagramPanel *panel = dynamic_cast<ModelDiagramPanel*>(reinterpret_cast<FormViewBase*>(arg_ptr));
      if (panel)
        panel->setup_navigator();
      break;
    }

  case wb::RefreshOverviewNodeInfo:
  {
    if (_model_panel && (arg_ptr == dynamic_cast<bec::UIForm*>(_model_panel->get_overview()->get_be()) || arg_ptr == NULL))
    {
      _model_panel->get_overview()->refresh_node(bec::NodeId(arg_id));
    }
    else
    {
      Gtk::Notebook *note= get_upper_note();
      for (int i= note->get_n_pages()-1; i >= 0; --i)
      {
//        Gtk::Widget *page= note->get_nth_page(i);
//        FormViewBase *view= static_cast<FormViewBase*>(page->get_data("FormViewBase"));
//        if (view && view->get_associated_view() && (arg_ptr == view->get_associated_view()->get_form()))
//        {
//          OverviewPanel *overview= dynamic_cast <OverviewPanel*> (view->get_associated_view()); 
//          overview->refresh_node(bec::NodeId(arg_id));
//          break;
//        }
      }
    }
    break;
  }
  
  case wb::RefreshOverviewNodeChildren:
  {
    if (_model_panel && (arg_ptr == dynamic_cast<bec::UIForm*>(_wbui_context->get_physical_overview()) || arg_ptr == NULL))
    {
      if (arg_id.empty())
      {
        _model_panel->get_overview()->show();
        _model_panel->get_overview()->rebuild_all();
      }
      _model_panel->get_overview()->refresh_children(bec::NodeId(arg_id));
    }
    else
    {
      Gtk::Notebook *note= get_upper_note();
      for (int i= note->get_n_pages()-1; i >= 0; --i)
      {
        Gtk::Widget *page= note->get_nth_page(i);
        FormViewBase *view= static_cast<FormViewBase*>(page->get_data("FormViewBase"));
/*        if (view && view->get_associated_view() && (arg_ptr == view->get_associated_view()->get_form()))
        {
          OverviewPanel *overview= dynamic_cast <OverviewPanel*> (view->get_associated_view()); 
          if (arg_id.empty())
          {
            overview->show();
            overview->rebuild_all();
          }
          overview->refresh_children(bec::NodeId(arg_id));
          break;
        }*/
      }
    }
    break;
  }
  
  case wb::RefreshViewName:
    {
      // Find tab with view and rename it
      FormViewBase *form= reinterpret_cast<FormViewBase*>(arg_ptr);
      Gtk::Notebook *note= get_upper_note();
      for (int c= note->get_n_pages(), i= 1; i < c; i++)
      {
        ModelDiagramPanel *model_panel = dynamic_cast<ModelDiagramPanel*>(reinterpret_cast<FormViewBase*>(note->get_nth_page(i)->get_data("FormViewBase")));

        if (model_panel && model_panel == form)
          model_panel->set_title(arg_id);
      }
      break;
    }
  case wb::RefreshDocument:
    {
      get_mainwindow()->set_title(_wbui_context->get_title());

      ///_model_sidebar.refresh_usertypes();
      break;
    }
  case wb::RefreshSelection:
    {
      FormViewBase *form = get_active_pane();
      
      if (form == _model_panel)
      {
        if (_model_panel)
          _model_panel->selection_changed();
      }
      else if (dynamic_cast<ModelDiagramPanel*>(form))
      {
        ModelDiagramPanel *model = dynamic_cast<ModelDiagramPanel*>(form);
        model->selection_changed();
      }
      break;
    }

  case wb::RefreshNewModel:
    handle_model_created();
    _wbui_context->get_wb()->new_model_finish();
    break;
    
  case wb::RefreshZoom:
    {
      ModelDiagramPanel *panel = dynamic_cast<ModelDiagramPanel*>(get_active_pane());
      if (panel)
        panel->refresh_zoom();
      break;
    }

  case wb::RefreshCloseEditor:
    _signal_close_editor.emit(arg_id);
    break;

  case wb::RefreshTimer:
    update_timer();
    break;
  case wb::RefreshFindOutput:
    break;

  case wb::RefreshFinishEdits:
    {
      Gtk::Widget *focused= get_mainwindow()->get_focus();
      if (focused)
      {
        if (dynamic_cast<Gtk::Entry*>(focused))
        {
          Gtk::TreeView *tree = dynamic_cast<Gtk::TreeView*>(focused->get_parent());
          if (!tree && focused->get_parent())
            tree = dynamic_cast<Gtk::TreeView*>(focused->get_parent()->get_parent());
          
          if (tree)
          {
            Gtk::TreePath path;
            Gtk::TreeViewColumn *column = 0;
            tree->get_cursor(path, column);
            if (column)
            {
              // this is not working...
              std::vector<Gtk::CellRenderer*> rends(column->get_cell_renderers());
              for (std::vector<Gtk::CellRenderer*>::iterator iter = rends.begin(); iter != rends.end(); ++iter)
              {
                (*iter)->stop_editing(true);
              }
              // fallback
              tree->set_cursor(path, *column, false);
            }
          }
        }
      }
    }
    break;
  }
}

//------------------------------------------------------------------------------

void MainForm::handle_model_created()
{
  wb::OverviewBE* overview_be = _wbui_context->get_physical_overview();
  // Create the model overview panel
  _model_panel= ModelPanel::create(_wbui_context, overview_be);
  _model_panel->set_close_editor_callback(sigc::bind(sigc::ptr_fun(close_plugin), _wbui_context->get_wb()));
  
  _model_panel->get_overview()->get_be()->set_frontend_data(dynamic_cast<FormViewBase*>(_model_panel));

  _signal_close_editor.connect(sigc::hide_return(sigc::mem_fun(_model_panel, &ModelPanel::close_editors_for_object)));

  add_form_pane(_model_panel);
  
  _model_panel->get_overview()->rebuild_all();
}
//------------------------------------------------------------------------------
void MainForm::handle_model_closed()
{
  if (_model_panel)
    get_upper_note()->remove_page(*_model_panel);
  delete _model_panel;
  _model_panel= 0;
}

//------------------------------------------------------------------------------
void MainForm::lock_gui_becb(bool lock)
{
  _gui_locked= lock;
}

//------------------------------------------------------------------------------
void MainForm::switch_page(GtkNotebookPage*, guint pagenum)
{
  Gtk::Notebook *note= get_upper_note();
  Gtk::Widget *page = note->get_nth_page(pagenum);
  if (page)
  {
    page->show();

    bec::UIForm* uiform = static_cast<bec::UIForm*>(page->get_data("uiform"));
    _wbui_context->set_active_form(uiform);
  }
  _wbui_context->get_command_ui()->signal_validate_edit_menu_items().emit();
}


//==============================================================================
ModelDiagramPanel *MainForm::get_panel_for_view(mdc::CanvasView *view)
{
  bec::UIForm *form;
  if ((form= _wbui_context->get_wb()->get_model_context()->get_diagram_form(view)))
    return dynamic_cast<ModelDiagramPanel*>(static_cast<FormViewBase*>(form->get_frontend_data()));

  return 0;
}

void MainForm::add_plugin_pane(PluginEditorBase *pane)
{
  // locate the active main tab
  Gtk::Notebook *note = get_upper_note(); 
  int pagenum= note->get_current_page();

  if (pagenum < 0)
  {
    g_warning("Cannot add plugin with no active tab");
    return;
  }
  
  Gtk::Widget *page = note->get_nth_page(pagenum);
  if (page)
  {
    FormViewBase *form = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
    
    if (form)
    {
      // add the editor to the active tab
      form->add_plugin_tab(pane);

      pane->set_data("ContainerForm", form);
    }
  }
}


void MainForm::bring_plugin_pane(PluginEditorBase *pane)
{
  // locate the active main tab
  Gtk::Notebook *note = get_upper_note(); 
  int pagenum= note->get_current_page();

  if (pagenum < 0)
  {
    g_warning("Cannot add plugin with no active tab");
    return;
  }
  

  FormViewBase *old_form = reinterpret_cast<FormViewBase*>(pane->get_data("ContainerForm"));

  Gtk::Widget *page = note->get_nth_page(pagenum);
  if (page)
  {
    FormViewBase *form = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
    
    if (form && old_form != form)
    {
      pane->reference();
      if (old_form)
        old_form->remove_plugin_tab(pane);
      // remove from where it is now
      // add the editor to the active tab
      form->add_plugin_tab(pane);
      pane->unreference();
      pane->set_data("ContainerForm", form);
    }
  }
}



static bool close_plugin_form(GdkEventAny *ev, PluginEditorBase *frame)
{
  frame->close_live_object_editor();
  return true;
}

void MainForm::add_plugin_form(PluginEditorBase *frame)
{
  Gtk::Window *window= new Gtk::Window();
  window->add(*frame);
  window->set_title(frame->get_title());
  window->signal_delete_event().connect(sigc::bind(sigc::ptr_fun(close_plugin_form), frame));
  frame->signal_title_changed().connect(sigc::mem_fun(window, &Gtk::Window::set_title));
  int width= 800;
  int height= 600;
  window->resize(width, height);
  window->show_all();
}


void MainForm::close_active_tab()
{  
  Gtk::Widget *focused= get_mainwindow()->get_focus();
  Gtk::Notebook *upper= get_upper_note();

  // go up the hierarchy to see if a child of the upper notebooks is focused
  while (focused && focused != upper)
    focused= focused->get_parent();

  if (focused)
  {
    int curpagenum = upper->get_current_page();
    if (curpagenum >= 0)
    {
      Gtk::Widget *page = upper->get_nth_page(curpagenum);
      FormViewBase *panel = reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
      bec::UIForm *form = reinterpret_cast<bec::UIForm*>(page->get_data("uiform"));
      if (form && form->get_form_context_name() == "home")
        return;
      
      if (panel)
      {
        if (!panel->close_focused_tab())
          close_tab(upper, page, sigc::mem_fun(panel, &FormViewBase::will_close));
      }
      else
        close_tab(upper, page, sigc::slot<bool>());
    }
  }
}


void MainForm::close_tab(Gtk::Notebook *note, Gtk::Widget *page, const sigc::slot<bool> &on_close)
{
  // on_close should return true if the form should be closed/removed
  if (on_close)
  {
    if (on_close())
    {
      bool is_appview = page->get_data("AppView") != 0;
      page->hide();
      note->remove_page(*page);
      if (!is_appview)
        page->unreference();
    }
    else
      return;
  }
  else
    page->hide();

  bool visible= false;
  for (int c= note->get_n_pages(), i= 0; i < c; i++)
  {
    if (note->get_nth_page(i)->is_visible())
    {
      visible= true;
      break;
    }
  }

  if (!visible)
    note->hide();
}


void MainForm::append_tab_page(Gtk::Notebook *note, Gtk::Widget *widget, const std::string &title,
                               bool auto_switch,
                               const sigc::slot<bool> &on_close,
                               Gtk::Label **title_label_ret)
{
  // Make a lookup of existing page with the same content
  bool already_have_page = false;
  for (int i = note->get_n_pages() - 1; i >= 0; --i)
  {
    if (note->get_nth_page(i) == widget)
    {
      already_have_page = true;
      break;
    }
  }

  if ( ! already_have_page )
  {
    Gtk::Widget *tab= 
      create_closeable_tab(title, sigc::bind(sigc::mem_fun(this, &MainForm::close_tab),
                                             note, widget, on_close),
                           title_label_ret);

    note->append_page(*widget, *tab);

    if (auto_switch)
    {
      // switch the current page when we're in idle
      Glib::signal_idle().connect(sigc::bind_return(sigc::bind(sigc::mem_fun(note, &Gtk::Notebook::set_current_page),
                                                               note->get_n_pages()-1), false));
    }
    note->show();
  }
}

/*
void MainForm::add_model_pane(ModelDiagramPanel *panel)
{
  bool is_closed= panel->get_diagram_form()->is_closed();
  
  panel->get_panel()->set_data("FormViewBase", dynamic_cast<FormViewBase*>(panel));
  panel->get_panel()->set_data("uiform", dynamic_cast<bec::UIForm*>(panel->get_form()));

  append_tab_page(get_upper_note(), panel->get_panel(), panel->get_title(), !is_closed);

  if (is_closed)
    panel->get_panel()->hide();

  Gtk::Label *label= (Gtk::Label*)*((Gtk::HBox*)get_upper_note()->get_tab_label(*panel->get_panel()))->get_children().begin();
  if (label)
    panel->signal_title_changed().connect(sigc::mem_fun(label, &Gtk::Label::set_text));

//  _toolbar_manager->rebuild_toolbar(toolbar, "tools");
}*/


FormViewBase *MainForm::get_active_pane()
{
  int p = get_upper_note()->get_current_page();
  if (p >= 0)
  {
    Gtk::Widget *page = get_upper_note()->get_nth_page(p);
    if (page)
      return reinterpret_cast<FormViewBase*>(page->get_data("FormViewBase"));
  }
  return 0;
}


void MainForm::add_form_pane(FormViewBase *panel, bool opened)
{
  panel->get_panel()->set_data("FormViewBase", panel);
  panel->get_panel()->set_data("uiform", dynamic_cast<bec::UIForm*>(panel->get_form()));

  append_tab_page(get_upper_note(), panel->get_panel(), panel->get_title(), opened,
                  sigc::mem_fun(panel, &FormViewBase::will_close));
  if (!opened)
    panel->get_panel()->hide();

  Gtk::Label *label= (Gtk::Label*)*((Gtk::HBox*)get_upper_note()->get_tab_label(*panel->get_panel()))->get_children().begin();
  if (label)
    panel->signal_title_changed().connect(sigc::mem_fun(label, &Gtk::Label::set_text));

  panel->reset_layout();
}


void MainForm::prepare_close_document()
{
  // close all diagram tabs so they stop receiving events (which can lead to a crash)
  Gtk::Notebook *note= get_upper_note();

  for (int i= note->get_n_pages()-1; i >= 0; --i)
  {
    Gtk::Widget *panel= note->get_nth_page(i);
    if (dynamic_cast<ModelDiagramPanel*>(panel))
      panel->hide();
  }

  // reset overview
  if (_model_panel)
    _model_panel->get_overview()->reset();
}


bool MainForm::fire_timer()
{
  _wbui_context->get_wb()->get_grt_manager()->flush_timers();

  update_timer();
  return false;
}


void MainForm::update_timer()
{
  int delay_ms= (int)(1000 * _wbui_context->get_wb()->get_grt_manager()->delay_for_next_timeout());

  if (delay_ms >= 0)
    Glib::signal_timeout().connect(sigc::mem_fun(this, &MainForm::fire_timer), delay_ms);
}



//==============================================================================
// Here go command handlers
//
//------------------------------------------------------------------------------
void MainForm::reset_layout()
{
///
}



void MainForm::show_about()
{
  Gtk::Window dialog(Gtk::WINDOW_TOPLEVEL);

  dialog.set_default_size(560, 322);
  dialog.modify_bg_pixmap(Gtk::STATE_NORMAL, bec::IconManager::get_instance()->get_icon_path(SPLASH_IMAGE));
  dialog.set_position(Gtk::WIN_POS_CENTER);
  dialog.set_decorated(false);

  Gtk::Fixed fixed;
  dialog.add(fixed);

  Gtk::Label label(base::strfmt("Version %s\nrev %i", PACKAGE_VERSION, APP_REVISION_NUMBER));
  label.set_alignment(1.0, 0.0);
  label.set_justify(Gtk::JUSTIFY_RIGHT);
  fixed.put(label, 10, 140);
  label.set_size_request(540, 50);
  label.modify_fg(Gtk::STATE_NORMAL, label.get_style()->get_black());

  dialog.set_modal(true);
  dialog.show_all();

  dialog.add_events(Gdk::BUTTON_PRESS_MASK);
  dialog.signal_button_press_event().connect_notify(sigc::hide(sigc::ptr_fun(&Gtk::Main::quit)));

  Gtk::Main::instance()->run();

  dialog.hide();
}


void MainForm::show_diagram_options()
{
  DiagramSizeForm *form= DiagramSizeForm::create(_wbui_context);

  form->run();

  form->hide();
  delete form;
}


void MainForm::show_find()
{
  if (!_find_panel)
  {
    _find_panel= FindPanel::create(_wbui_context);

    Gtk::VBox *vbox;
    _ui->get("inner_vbox", &vbox);

    vbox->pack_start(*_find_panel, false, false);
    vbox->reorder_child(*_find_panel, 0);
    _find_panel->unreference();

  }

  _find_panel->show();
}



void MainForm::show_page_setup()
{
  _wbui_context->get_wb()->execute_plugin("wb.print.setup");
}




#include <mforms.h>
#include "gtk/lf_view.h"

static std::string get_resource_path(mforms::App* app, const std::string& file)
{
  MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());
  if (file.empty()) return self->get_wbui_context()->get_wb()->get_grt_manager()->get_data_file_path("");
  if (file[0] == '/') return file;

  if (g_str_has_suffix(file.c_str(), ".png") || g_str_has_suffix(file.c_str(), ".xpm"))
    return bec::IconManager::get_instance()->get_icon_path(file);
  else if (g_str_has_suffix(file.c_str(), ".vbs"))
  {
    return self->get_wbui_context()->get_wb()->get_grt_manager()->get_data_file_path(file);
  }

  return self->get_wbui_context()->get_wb()->get_grt_manager()->get_data_file_path(file);
}


static std::string get_main_window_bounds(mforms::App* app)
{
  Gtk::Window *w = get_mainwindow();
  int x, y;
  
  w->get_window()->get_position(x, y);
  
  return base::strfmt("%i %i %i %i", x, y, w->get_width(), w->get_height());
}


void MainForm::set_status_text(mforms::App* app, const std::string &text)
{
  MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());
  
  self->show_status_text_becb(text);
}


static void begin_thread_loop()
{
}

static void end_thread_loop()
{
}


static std::list<std::pair<int,int> > event_loop_exit_codes;

static int begin_event_loop(mforms::App*,float timeout)
{
  Gtk::Main::run();
  if (event_loop_exit_codes.empty() || event_loop_exit_codes.back().first != (int)Gtk::Main::level())
  {
    // means something other than end_event_loop() called quit(), could mean the app is being quit.. 
    Gtk::Main::quit();
    return -1;
  }
  else
  {
    int rc = event_loop_exit_codes.back().second;
    event_loop_exit_codes.pop_back();
    return rc;
  }
}

static void end_event_loop(mforms::App *,int rc)
{
  Gtk::Main::quit();
  event_loop_exit_codes.push_back(std::make_pair(Gtk::Main::level()-1, rc));
}

void MainForm::setup_mforms_app()
{
  mforms::ControlFactory *cf = mforms::ControlFactory::get_instance();
  g_assert(cf);
  
  mforms::App::get()->set_data(this);
  
  cf->_app_impl.dock_view= &MainForm::dock_view;
  cf->_app_impl.select_view= &MainForm::select_view;
  cf->_app_impl.undock_view= &MainForm::undock_view;
  cf->_app_impl.set_view_title= &MainForm::set_view_title;
  cf->_app_impl.get_resource_path = &get_resource_path;
  cf->_app_impl.set_status_text= &set_status_text;
  cf->_app_impl.get_bounds= &get_main_window_bounds;
  cf->_app_impl.begin_thread_loop= &begin_thread_loop;
  cf->_app_impl.end_thread_loop= &end_thread_loop;
  cf->_app_impl.enter_event_loop= &begin_event_loop;
  cf->_app_impl.exit_event_loop= &end_event_loop;
}



Gtk::Widget *MainForm::decorate_widget(Gtk::Widget *panel, bec::UIForm *form)
{
  mforms::MenuBar *menu = form->get_menubar();
  mforms::ToolBar *toolbar = form->get_toolbar();

  Gtk::VBox *top_box = Gtk::manage(new Gtk::VBox(false, 0));
  if (menu)
  {
    Gtk::Widget *w = mforms::gtk::widget_for_menubar(menu);
    top_box->pack_start(*w, false, true);
    w->show();
  }

  if (toolbar)
  {
    Gtk::Widget *w = mforms::gtk::widget_for_toolbar(toolbar);
    top_box->pack_start(*w, false, true);
    w->show();
  }
  top_box->pack_start(*panel, true, true);
  panel->show();

  top_box->show();


  return top_box;
}


static gpointer delete_appview(mforms::AppView *appview)
{
  delete appview;
  return 0;
}

void MainForm::dock_view(mforms::App *app, mforms::AppView *view, const std::string &position)
{
  MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());
  g_return_if_fail(view != NULL);
  Gtk::Widget *w= mforms::gtk::ViewImpl::get_widget_for_view(view);
  g_return_if_fail(w != NULL);

  if (position == "maintab")
  {
    Gtk::Notebook *note = self->get_upper_note();
    // Make a lookup of existing page with the same content
    bool already_have_page = false;
    for (int i = note->get_n_pages() - 1; i >= 0; --i)
    {
      Gtk::Widget *page;
      if ((page = note->get_nth_page(i)) && page->get_data("uiform")  == dynamic_cast<bec::UIForm*>(view))
      {
        already_have_page = true;
        break;
      }
    }
    if (already_have_page)
      return;
    
    if (!view->get_menubar())
    {
      view->set_menubar(self->_wbui_context->get_command_ui()->create_menubar_for_context(""));
      view->get_menubar()->set_managed();
      view->get_menubar()->release();
    }

    Gtk::Widget *decorated = self->decorate_widget(w, view);

    decorated->set_data("AppView", view);
    decorated->add_destroy_notify_callback(view, (gpointer(*)(gpointer))delete_appview);

    if (dynamic_cast<bec::UIForm*>(view)->get_form_context_name() == "home")
    {
      Gtk::Widget *tab = Gtk::manage(new Gtk::Image(bec::IconManager::get_instance()->get_icon_path("WB_Home.png")));
      Gtk::Alignment *align = Gtk::manage(new Gtk::Alignment());
      align->add(*tab);
      align->show_all();
      align->set_padding(0, 0, 10, 10);
      note->append_page(*decorated, *align);
    }
    else
    {
      Gtk::Label *label= 0;
      self->append_tab_page(self->get_upper_note(), decorated, "", true,
                            sigc::mem_fun(view, &mforms::AppView::on_close),
                            &label);    
      w->set_data("tablabel", label);
    }
    decorated->set_data("uiform", dynamic_cast<bec::UIForm*>(view));
    w->set_data("containerNote", note);
  }
  else if (position == "editortab")
  {
    Gtk::Label *label= 0;
    
///    self->append_tab_page(self->get_lower_note(), w, "", true,
///                          sigc::mem_fun(view, &mforms::AppView::on_close),
///                          &label);
    w->set_data("tablabel", label);
///    w->set_data("containerNote", self->get_lower_note());
  }
}


bool MainForm::select_view(mforms::App *app, const std::string &id)
{
  MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());

  Gtk::Notebook *upper_note = self->get_upper_note();
  for (int i= 0; i < upper_note->get_n_pages(); i++)
  {
    Gtk::Widget *page = upper_note->get_nth_page(i);
    if (page)
    {
      mforms::AppView *appv = dynamic_cast<mforms::AppView*>(reinterpret_cast<bec::UIForm*>(page->get_data("uiform")));
      if (appv->identifier() == id)
      {
        upper_note->set_current_page(i);
        return true;
      }
    }
  }
  return false;
}


void MainForm::undock_view(mforms::App *app, mforms::AppView *view)
{
  MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());
  g_return_if_fail(view != NULL);
  Gtk::Widget *w= mforms::gtk::ViewImpl::get_widget_for_view(view);
  g_return_if_fail(w != NULL);
  
  Gtk::Notebook *note= reinterpret_cast<Gtk::Notebook*>(w->get_data("containerNote"));
  
  if (note)
    note->remove_page(*w);
}


void MainForm::set_view_title(mforms::App *app, mforms::AppView *view, const std::string &title)
{
  MainForm *self= reinterpret_cast<MainForm*>(app->get_data_ptr());
  g_return_if_fail(view != NULL);
  Gtk::Widget *w= mforms::gtk::ViewImpl::get_widget_for_view(view);
  g_return_if_fail(w != NULL);

  Gtk::Label *label= reinterpret_cast<Gtk::Label*>(w->get_data("tablabel"));
  if (label)
    label->set_text(title);
}

